/**
 * JASMINe
 * Copyright (C) 2005-2007 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: EntitySampler.java 55 2007-12-17 16:03:02Z tokmensa $
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.monitoring.mbeancmd.sampling;

import java.io.IOException;
import java.util.Iterator;

import javax.management.Attribute;
import javax.management.AttributeList;
import javax.management.InstanceNotFoundException;
import javax.management.MBeanServerConnection;
import javax.management.ObjectName;
import javax.management.ReflectionException;

/**
 * Sampler for entity beans.
 */
public class EntitySampler extends Sampler<XSampleData> {
    /**
     * Implementation of inherited method.
     *
     * @see Sampler#Sampler(String)
     */
    public EntitySampler(final String urlString) {
        super(urlString);
    }

    /**
     * Implementation of inherited abstract method.
     *
     * @see Sampler#getDefaultOn()
     */
    public String getDefaultOn() {
        return "*:j2eeType=EntityBean,*";
    }

    /**
     * Implementation of inherited abstract method.
     *
     * @see Sampler#newSampleData()
     */
    protected XSampleData newSampleData() {
        return new XSampleData();
    }

    /**
     * Implementation of inherited abstract method.
     *
     * @see Sampler#poll(SampleData)
     * @see EntitySampler#pollEntities(XSampleData)
     */
    protected XSampleData poll(final XSampleData data) {
        pollEntities(data);
        data.setValid(true);
        return data;
    }

    /**
     * Polls entity providers for data, will call
     * {@link EntitySampler#pollEntity(ObjectName, EntityData)}
     * for each object name.
     *
     * @param data  XSampleData to add polled information to.
     */
    private void pollEntities(final XSampleData data) {
        for (int i = 0; i < oNames.length ; i++) {
            EntityData entdata = new EntityData();
            entdata.setObjectName(oNames[i]);
            entdata.setSampleTime(System.currentTimeMillis());
            pollEntity(oNames[i], entdata);

            if (entdata.isValid()) {
                data.put(oNames[i], entdata);
            }
        }
    }

    /**
     * Polls one entity provider for data.
     *
     * @param on    Name of the provider.
     * @param data  EntityData to add polled information to.
     */
    private void pollEntity(final ObjectName on, final EntityData data) {
        AttributeList al = null;
        MBeanServerConnection cnx = getMBeanServerConnection();
        Attribute att;
        try {
            al = cnx.getAttributes(on,attIds);
            Iterator it = al.iterator();
            while (it.hasNext()) {
                att = (Attribute) it.next();
                data.setAttribute(att);
            }
            data.setServerInfo(getServerName(), getServerDomain());
            data.setValid(true);
        } catch (InstanceNotFoundException e) {
            e.printStackTrace(System.err);
        } catch (ReflectionException e) {
            e.printStackTrace(System.err);
        } catch (IOException e) {
            e.printStackTrace(System.err);
        }
    }


    /**
     * List of polled attributes :
     *
     *      - PoolSize : Current nb of free instances.
     *      - CacheSize : Total nb of instances in memory.
     *
     *      - Entity Counters :
     *          0: Used in TX  : Current nb of instances used inside transactions.
     *          1: Used outside TX : Current nb of instances used outside transaction.
     *          2: Unused but ready : Current nb of unused, ready instances.
     *          3: Marked removed : Current nb of instances marked removed.
     *          4: PK number : Total nb of PK in memory.
     *          5: Passivated : Current nb of passivated instances.
     */
    private static final String[] attIds = {
             "name",
             "cacheSize",
             "poolSize",
             "entityCounters",
             "maxCacheSize",
             "minPoolSize"
    };
}
