/**
 * JASMINe
 * Copyright (C) 2005-2007 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: Sampler.java 55 2007-12-17 16:03:02Z tokmensa $
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.monitoring.mbeancmd.sampling;

import java.io.PrintStream;
import java.util.Set;

import javax.management.MBeanServerConnection;
import javax.management.ObjectName;

import org.ow2.jasmine.monitoring.mbeancmd.JmxAP;

/**
 * Base class for all sampler classes.
 */
public abstract class Sampler<T extends SampleData> {
    /**
     * Creates the Sampler using a JMX URL.
     *
     * @param urlString  JMX URL.
     */
    public Sampler(final String urlString) {
        jmxUrl = urlString;
    }

    /**
     * @return  Connection to MBean server. Will return existing instance if
     *          already connected, and try to connect if no connection exists.
     *          Returns null on failure.
     */
    protected final MBeanServerConnection getMBeanServerConnection() {
        if (mbscnx == null) {
            try {
                JmxAP jmxap = null;
                if (jmxUrl == null) {
                    jmxap = new JmxAP();
                } else {
                    jmxap = new JmxAP(jmxUrl);
                }
                mbscnx = jmxap.getMBeanServerConnection();
                if (serverDomain == null) {
                    String[] managees = JmxAP.getJmxTargets();
                    String url = jmxap.getJmxUrlASString();
                    for (int i = 0; i < managees.length; i++) {
                        if (JmxAP.getJmxUrl(managees[i]).equals(url)) {
                            serverName = managees[i];
                            ObjectName j2eeinstance = ObjectName.getInstance("*:j2eeType=J2EEServer,*");
                            Set<ObjectName> son = mbscnx.queryNames(j2eeinstance, null);
                            serverDomain = son.iterator().next().getDomain();
                            break;
                        }
                    }
                }
            } catch (Exception ex) {
                ex.printStackTrace(System.err);
            }
        }
        return mbscnx;
    }

    /**
     * Gets the list of object names that match a given object name prototype.
     *
     * @param onProto  Object name filtering pattern to use.
     *
     * @return  All objects that match onProto.
     */
    protected final ObjectName[] queryON(final String onProto) {
        ObjectName[] ons = null;
        try {
            Set<ObjectName> onSet = null;
            MBeanServerConnection cnx = getMBeanServerConnection();

            onSet = cnx.queryNames(new ObjectName(onProto), null);
            if (!onSet.isEmpty()) {
                ons = onSet.toArray(new ObjectName[onSet.size()]);
                for (int i = 0; i < ons.length; i++) {
                    System.out.println("Found ObjectName in "
                            + serverDomain + "." + serverName + ": "
                            + ons[i]);
                }
            }
        } catch (Exception e) {
            e.printStackTrace(System.err);
        }
        return ons;
    }

    /**
     * return  Default Object name filtering pattern for this sampler. If null,
     *         then we'll consider that the Sampler doesn't have one default
     *         object name filter.
     */
    protected abstract String getDefaultOn();

    /**
     * return  Object name filtering pattern.
     */
    protected String getOn() {
        return this.objectName;
    }

    /**
     * @param onProto  Object name filtering pattern to set.
     */
    public void setOn(final String onProto) {
        this.objectName = onProto;
    }

    /**
     * Initialize the sampler, itself called by
     * {@link Sampler#process(long, PrintStream)}.
     *
     * This implementation will fill {@link Sampler#oNames} based on the
     * object name filter set using {@link Sampler#setOn(String)}.
     *
     * If no filter was set before calling this method, will call set object
     * name to {@link Sampler#getDefaultOn()}.
     *
     * If {@link Sampler#getDefaultOn()} has returned null, then
     * {@link Sampler#oNames} will remain null.
     */
    public void init() {
        if(getOn()==null) {
            setOn(getDefaultOn());
        }

        if(getOn()!=null) {
            oNames = queryON(getOn());
        }
    }

    /**
     * Samples data based on a previous sample.
     *
     * @param previous  Previous sample.
     *
     * @return  Sampled data, @see SampleData#compute(SampleData)
     */
    public final T sample(final T previous) {
        T data = newSampleData();

        try {
            data.setSampleTime(System.currentTimeMillis());
            poll(data);
            if (data.isValid()) {
                data.compute(previous);
            }
        } catch (Exception e) {
            e.printStackTrace(System.err);
        }

        if (data.isValid()) {
            return data;
        } else {
            return null;
        }
    }

    /**
     * Start polling data. Will never return (except on failure).
     *
     * @param period  Poll period.
     * @param out     Stream to write results into.
     */
    public final void process(final long period, final PrintStream out) {
        boolean started = false;
        T d0 = null;
        T data = null;

        init();

        boolean printHeader = true;
        while (true) {
            try {
                data = sample(d0);
                if (data != null) {
                    if (printHeader) {
                        out.println(data.getPrintHeader());
                        printHeader = false;
                    }
                    data.printData(out);
                }

                if (!started) {
                    started = true;
                }
            } catch (Exception e) {
                e.printStackTrace(System.err);
                if (started) {
                    System.err.println("End polling");
                    break;
                }
            }

            d0 = data;
            try {
                Thread.sleep(period * 1000);
            } catch (Exception e) {
                // Nothing
            }
        }
    }

    /**
     * @return  Server name.
     */
    public String getServerName() {
        return serverName;
    }

    /**
     * @return  Server's domain name.
     */
    public String getServerDomain() {
        return serverDomain;
    }

    /**
     * @return  New sample data.
     */
    protected abstract T newSampleData();

    /**
     * Poll based on a previous sample.
     *
     * @param data  Previous sample.
     *
     * @return  Resulting data.
     */
    protected abstract T poll(final T data);

    /**
     * Object names, see {@link Sampler#init()}
     */
    protected ObjectName[] oNames = null;

    /**
     * Object name.
     */
    private String objectName = null;

    /**
     * Server name, set automatically when
     * {@link Sampler#getMBeanServerConnection()} is called.
     */
    private String serverName = null;

    /**
     * Server's domain name, set automatically when
     * {@link Sampler#getMBeanServerConnection()} is called.
     */
    private String serverDomain = null;

    /**
     * JMX URL.
     */
    private String jmxUrl = null;

    /**
     * Connection to MBean server.
     */
    private MBeanServerConnection mbscnx = null;
}
